﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CtrEditor.Convertidores;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucTransporteGuias.xaml
    /// </summary>
    public class osTransporteGuias : osBase
    {
        private string _nombre = "Transporte Guias";

        private float frictionCoefficient;
        private float velMax50hz;  // en metros por minuto      
        private float tiempoRampa;
        private bool esMarcha;
        private double _distance;
        private float altoGuia;

        private float left;
        private float top;

        private Rectangle TransporteCentral = new Rectangle();
        private Line Guia_Superior = new Line();
        private Line Guia_Inferior = new Line();

        public override float Left
        {
            get => left;
            set
            {
                left = value;
                CanvasSetLeftinMeter(value);
                OnPropertyChanged(nameof(Left));
            }
        }
        public override float Top
        {
            get => top;
            set
            {
                top = value;
                CanvasSetTopinMeter(value);
                OnPropertyChanged(nameof(Top));
            }
        }

        public float Ancho
        {
            get => TransporteCentral.Length;
            set
            {
                TransporteCentral.Length = value;
                OnPropertyChanged(nameof(Ancho));
            }
        }
        public float AltoGuia
        {
            get => altoGuia;
            set
            {
                altoGuia = value;
                OnPropertyChanged(nameof(AltoGuia));
            }
        }

        public float Alto
        {
            get => TransporteCentral.Width;
            set
            {
                TransporteCentral.Width = value;
                OnPropertyChanged(nameof(Alto));
            }
        }

        public float Angulo
        {
            get => TransporteCentral.Angle;
            set
            {
                TransporteCentral.Angle = value;
                OnPropertyChanged(nameof(Angulo));
            }
        }
        public float VelocidadActual
        {
            get => TransporteCentral.Speed;
            set
            {
                TransporteCentral.Speed = value;
                OnPropertyChanged(nameof(VelocidadActual));
            }
        }

        public double Distance
        {
            get { return _distance; }
            set
            {
                if (_distance != value)
                {
                    _distance = value;
                    OnPropertyChanged("Distance");
                }
            }
        }

        public override string Nombre
        {
            get => _nombre;
            set
            {
                if (_nombre != value)
                {
                    _nombre = value;
                    OnPropertyChanged(nameof(Nombre));
                }
            }
        }

        private void ActualizarGeometrias()
        {
            ucTransporteGuias ucTG = (ucTransporteGuias)_visualRepresentation;
            if (ucTG != null)
            {
                var _canvasLeft = CanvasGetLeftinMeter();
                var _canvasTop = CanvasGetTopinMeter();

                var coordenadas = GetRectangleCoordinatesInMeter(ucTG.Transporte, ucTG);
                TransporteCentral.Left = coordenadas.Left + _canvasLeft;
                TransporteCentral.Top = coordenadas.Top + _canvasTop;

                coordenadas = GetRectangleCoordinatesInMeter(ucTG.GuiaSuperior, ucTG);
                Guia_Superior.Left = coordenadas.Left + _canvasLeft;
                Guia_Superior.Top = coordenadas.Top + _canvasTop; ;

                coordenadas = GetRectangleCoordinatesInMeter(ucTG.GuiaInferior, ucTG);
                Guia_Inferior.Left = coordenadas.Left + _canvasLeft;
                Guia_Inferior.Top = coordenadas.Top + _canvasTop; ;

                TransporteCentral.Angle = Guia_Superior.Angle = Guia_Inferior.Angle = Angulo;
                Guia_Superior.Length = Guia_Inferior.Length = Ancho;
            }
        }

        private (float Left, float Top) GetRectangleCoordinatesInMeter(System.Windows.Shapes.Rectangle rect, ucTransporteGuias ucTG)
        {
            if (rect != null)
            {
                // Obtiene la transformada del objeto visual
                GeneralTransform transform = rect.TransformToAncestor(ucTG);

                // Obtiene la posición absoluta
                Point topLeft = transform.Transform(new Point(0, 0));
                //Point bottomRight = transform.Transform(new Point(rect.ActualWidth, rect.ActualHeight));
                return (PixelToMeter.Instance.calc.PixelsToMeters((float)topLeft.X), PixelToMeter.Instance.calc.PixelsToMeters((float)topLeft.Y));
            }
            else return (0,0);
        }



        public float FrictionCoefficient { get => frictionCoefficient; set => frictionCoefficient = value; }
        public float VelMax50hz { get => velMax50hz; set => velMax50hz = value; }
        public float TiempoRampa { get => tiempoRampa; set => tiempoRampa = value; }
        public bool EsMarcha { get => esMarcha; set => esMarcha = value; }

        public osTransporteGuias()
        {
            Ancho = 1;
            Alto = 0.10f;
            AltoGuia = 0.03f;
            Distance = 0.01f;
        }

        public override void ConnectSimManager(SimulationManager simulationManager)
        {
            simulationManager.rectangles.Add(TransporteCentral);
            simulationManager.lines.Add(Guia_Superior);
            simulationManager.lines.Add(Guia_Inferior);
        }
        public override void UpdateGeometry()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }

        public override void UpdateControl()
        {
        }


    }

    public partial class ucTransporteGuias : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucTransporteGuias()
        {
            InitializeComponent();
        }
        public void Resize(float width, float height)
        {
            if (Datos is osTransporteGuias datos)
                datos.Ancho = PixelToMeter.Instance.calc.PixelsToMeters(width);
        }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle)
        {
            if (Datos != null)
                if (Datos is osTransporteGuias datos)
                    datos.Angulo = Angle;
        }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 1;
        }

    }
}
